# ===== THIS FILE IS GENERATED FROM A TEMPLATE ===== #
# ============== DO NOT EDIT DIRECTLY ============== #
# pylint: disable=W0201

from typing import Optional  # pylint: disable=unused-import
from ..protobufs import main_pb2
from .pvt_axis_type import PvtAxisType


class PvtAxisDefinition:
    """
    Defines an axis of the PVT sequence.
    """

    def __init__(
            self: 'PvtAxisDefinition',
            axis_number: int,
            axis_type: Optional[PvtAxisType] = None
    ) -> None:
        self._axis_number = axis_number
        self._axis_type = axis_type

    @property
    def axis_number(self) -> int:
        """
        Number of a physical axis or a lockstep group.
        """

        return self._axis_number

    @axis_number.setter
    def axis_number(self, value: int) -> None:
        self._axis_number = value

    @property
    def axis_type(self) -> Optional[PvtAxisType]:
        """
        Defines the type of the axis.
        """

        return self._axis_type

    @axis_type.setter
    def axis_type(self, value: Optional[PvtAxisType]) -> None:
        self._axis_type = value

    def __repr__(self) -> str:
        return str(self.__dict__)

    @staticmethod
    def from_protobuf(
        pb_data: main_pb2.StreamAxisDefinition
    ) -> 'PvtAxisDefinition':
        instance = PvtAxisDefinition.__new__(
            PvtAxisDefinition
        )  # type: PvtAxisDefinition
        instance.axis_number = pb_data.axis_number
        instance.axis_type = PvtAxisType(pb_data.axis_type)
        return instance

    @staticmethod
    def to_protobuf(source: 'Optional[PvtAxisDefinition]') -> main_pb2.StreamAxisDefinition:
        pb_data = main_pb2.StreamAxisDefinition()

        if source is None:
            return pb_data

        if not isinstance(source, PvtAxisDefinition):
            raise TypeError("Provided value is not PvtAxisDefinition.")

        pb_data.axis_number = source.axis_number
        pb_data.axis_type = source.axis_type.value if source.axis_type is not None else 0
        return pb_data



# ===== THIS FILE IS GENERATED FROM A TEMPLATE ===== #
# ============== DO NOT EDIT DIRECTLY ============== #
# pylint: disable=W0201

from typing import List  # pylint: disable=unused-import
from ..protobufs import main_pb2


class MovementInterruptedExceptionData:
    """
    Contains additional data for MovementInterruptedException.
    """

    @property
    def warnings(self) -> List[str]:
        """
        The full list of warnings.
        """

        return self._warnings

    @warnings.setter
    def warnings(self, value: List[str]) -> None:
        self._warnings = value

    @property
    def reason(self) -> str:
        """
        The reason for the Exception.
        """

        return self._reason

    @reason.setter
    def reason(self, value: str) -> None:
        self._reason = value

    @property
    def device(self) -> int:
        """
        The address of the device that caused the interruption.
        """

        return self._device

    @device.setter
    def device(self, value: int) -> None:
        self._device = value

    @property
    def axis(self) -> int:
        """
        The number of the axis that caused the interruption.
        """

        return self._axis

    @axis.setter
    def axis(self, value: int) -> None:
        self._axis = value

    def __repr__(self) -> str:
        return str(self.__dict__)

    @staticmethod
    def from_protobuf(
        pb_data: main_pb2.MovementInterruptedExceptionData
    ) -> 'MovementInterruptedExceptionData':
        instance = MovementInterruptedExceptionData.__new__(
            MovementInterruptedExceptionData
        )  # type: MovementInterruptedExceptionData
        instance.warnings = list(pb_data.warnings)
        instance.reason = pb_data.reason
        instance.device = pb_data.device
        instance.axis = pb_data.axis
        return instance



# ===== THIS FILE IS GENERATED FROM A TEMPLATE ===== #
# ============== DO NOT EDIT DIRECTLY ============== #

from typing import Union
from .pvt_movement_failed_exception_data import PvtMovementFailedExceptionData
from ..protobufs import main_pb2
from .motion_lib_exception import MotionLibException


class PvtMovementFailedException(MotionLibException):
    """
    Thrown when a device registers a fault during PVT movement.
    """

    @property
    def details(self) -> PvtMovementFailedExceptionData:
        """
        Additional data for PvtMovementFailedException
        """
        return self._details

    def __init__(self, message: str, custom_data: Union[bytes, PvtMovementFailedExceptionData]):
        MotionLibException.__init__(self, message)

        if isinstance(custom_data, PvtMovementFailedExceptionData):
            self._details = custom_data
        else:
            protobuf_obj = main_pb2.PvtMovementFailedExceptionData()
            protobuf_obj.ParseFromString(custom_data)
            self._details = PvtMovementFailedExceptionData.from_protobuf(protobuf_obj)



from .process_controller import ProcessController as ProcessController
from .process import Process as Process
from .process_controller_source import ProcessControllerSource as ProcessControllerSource
from .process_controller_source_sensor import ProcessControllerSourceSensor as ProcessControllerSourceSensor
from .process_controller_mode import ProcessControllerMode as ProcessControllerMode



from ctypes import c_void_p, c_int, c_int64, c_uint8, CFUNCTYPE
from typing import Any
import platform
import sys
import importlib
from .version import __version__


def _load_library() -> Any:
    is64bit = sys.maxsize > 2**32
    os_system = platform.system().lower()
    os_machine = platform.machine().lower()

    if os_system == "darwin":
        arch = "uni"
    else:
        if os_machine.startswith("aarch64") or os_machine.startswith("arm"):
            arch = "arm64" if is64bit else "arm"
        else:
            arch = "amd64" if is64bit else "386"

    ext = ""
    if os_system == "linux":
        ext = ".so"
    if os_system == "darwin":
        ext = ".dylib"

    lib_name = f"zaber-motion-lib-{os_system}-{arch}{ext}"
    module_name = f"zaber_motion_bindings_{os_system}"
    binding_module = importlib.import_module(module_name)
    if __version__ != binding_module.__version__:
        raise RuntimeError((
            f"The dependent module {module_name} is not compatible with "
            f"zaber_motion ({__version__} != {binding_module.__version__}). "
            f"Please install the correct version {module_name}=={__version__}."))
    return binding_module.load_library(lib_name)


lib = _load_library()

CALLBACK = CFUNCTYPE(None, c_void_p, c_int64)

c_call = lib.call
c_call.argtypes = [c_void_p, c_int64, CALLBACK, c_uint8]
c_call.restype = c_int

c_set_event_handler = lib.setEventHandler
c_set_event_handler.argtypes = [c_int64, CALLBACK]
c_set_event_handler.restype = None
